#include "ieee1451_ncap.h"
#include "ieee1451_objects.h"
#include <utility/ostream.h>
#include "utility/string.h"

__USING_SYS
OStream cout;

class IEEE1451_NCAP_Application //: public IEEE1451_NCAP::Listener
{
public:
    IEEE1451_NCAP_Application();
    ~IEEE1451_NCAP_Application();
	


/*private:
    static IEEE1451_NCAP_Application *_application;*/
public: 
	 void read_temperature(TIM_Cache *tim_cache, unsigned long offset, const char *buffer, unsigned short length)
    IEEE1451_NCAP *ncap; 
	void execute();
	void report_tim_connected(const char *address);
	void update_tim(const char *address);
};

class IEEE1451_TEDS_Retriever
{
private:
    friend class IEEE1451_NCAP_Application;

    enum State
    {
        meta_teds = 0,
        tim_transducer_name_teds = 1,
        phy_teds = 2,
        transducer_channel_teds = 3,
        transducer_name_teds = 4,
        units_extension_teds = 5
    };

public:
    IEEE1451_TEDS_Retriever(const char *address, IEEE1451_NCAP_Application *application);
    void process(const char *message, unsigned int length);
    void execute();

private:
    IEEE1451_NCAP_Application *_application;
    short _state;
    char _teds_id;
    unsigned short _last_trans_id;
};


//--------------------------------------------

IEEE1451_NCAP_Application::IEEE1451_NCAP_Application()
{
	cout<<"ncap application\n";
	ncap = new IEEE1451_NCAP(); 
}

IEEE1451_NCAP_Application::~IEEE1451_NCAP_Application()
{
}

void IEEE1451_NCAP_Application:: execute() {
	cout<<"Execute\n";
        ncap->execute();
}

void IEEE1451_NCAP_Application::report_tim_connected(const char *address)
{
       update_tim(address);
}
void IEEE1451_NCAP_Application::update_tim(const char *address)
{
    IEEE1451_TEDS_Retriever *retriever = get_retriever(address);
    if (!retriever)
    {
        retriever = new IEEE1451_TEDS_Retriever(address, this);
        _retrievers.push_back(retriever);
    }else
        cout << "Retriever already exist in update_tim\n";
    retriever->execute();
}
IEEE1451_TEDS_Retriever *IEEE1451_NCAP_Application::get_retriever(unsigned short trans_id)
{
    std::list<IEEE1451_TEDS_Retriever *>::iterator it = _retrievers.begin();
    while (it != _retrievers.end())
    {
        IEEE1451_TEDS_Retriever *retriever = *it++;
        if (retriever->_last_trans_id == trans_id)
            return retriever;
    }

    return 0;
}

IEEE1451_TEDS_Retriever *IEEE1451_NCAP_Application::get_retriever(const char *address)
{
    std::list<IEEE1451_TEDS_Retriever *>::iterator it = _retrievers.begin();
    while (it != _retrievers.end())
    {
        IEEE1451_TEDS_Retriever *retriever = *it++;
        if (retriever->_tim->_address == address)
            return retriever;
    }

    return 0;
}
void IEEE1451_NCAP_Application::read_temperature(TIM_Cache *tim_cache, unsigned long offset, const char *buffer, unsigned short length)
{
    IEEE1451_TEDS_NCAP *channel = tim_cache->_tim->_transducer->get_teds(TEDS_TRANSDUCER_CHANNEL);

    char *sample_str = channel->get_tlv(TEDS_TRANSDUCER_CHANNEL_SAMPLE);
    IEEE1451_TEDS_NCAP *sample = new IEEE1451_TEDS_NCAP(sample_str[0], &sample_str[2], sample_str[1], true);

    char *data_model = sample->get_tlv(TEDS_TRANSDUCER_CHANNEL_DAT_MODEL);
    unsigned short model = data_model[2];

    delete sample;

    if (model == 1) //Model = 1 -> Float32: Single-precision real
    {
        char *data;

        char aux[11];
        itoa(offset, aux);
        data = aux;
        data += " ";

        for (unsigned short i = 0, j = 0; i < tim_cache->_tim->_transducer->_repeats; i++, j += 4)
        {
            float value = *((float *)(&buffer[j]));

            itoa((int) value, aux);
            data += aux;
            if (i != (tim_cache->_tim->_transducer->_repeats - 1))
                data += ",";
        }

        cout << "Read temperature (" << std::dec << tim_cache->_tim->_transducer->_repeats << "): " << data << "\n";

    }
}

void IEEE1451_NCAP_Application::update_tim(const std::string &address)
{
    IEEE1451_TEDS_Retriever *retriever = get_retriever(address);
    if (!retriever)
    {
        retriever = new IEEE1451_TEDS_Retriever(address, this);
        _retrievers.push_back(retriever);
    }else
        std::cout << "Retriever already exist in update_tim\n";
    retriever->execute();
}

//----------------------------------------


int main()
{
    cout << "+++++ Starting ncap +++++\n";

    IEEE1451_NCAP_Application *app = new IEEE1451_NCAP_Application();
    app->execute();
    return 0;
}
