//#define USE_SIP
#include "timer.h"
#include "ieee1451_ncap.h"
#include "ieee1451_objects.h"

//#include "../commons.h"
//#include "../g726.h"
#include <list>
#include <cstdlib>
/*
#ifdef USE_SIP
    #include "../sip/sip_defs.h"
    #include "../sip/sip_manager.h"
    #include "../sip/sip_user_agent.h"
    #include "../sip/rtp.h"
#endif
*/
class IEEE1451_TEDS_Retriever;

class IEEE1451_NCAP_Application : public IEEE1451_NCAP::Listener
{
private:
    struct TIM_Cache
    {
        IEEE1451_TIM_Channel *_tim;

        short *_audio_buffer;
        char _law;
        short _rate;
        short _reset;
   //     G726_state _state;
/*
#ifdef USE_SIP
        SIP_User_Agent *_ua;
        RTP_Adapter *_rtp;
        void *_rtp_mutex;

        TIM_Cache(IEEE1451_TIM_Channel *tim, SIP_User_Agent *ua);
#else */
        void *_read_thread;

        TIM_Cache(IEEE1451_TIM_Channel *tim);
//#endif
        ~TIM_Cache();
    };

    IEEE1451_NCAP_Application();

public:
    ~IEEE1451_NCAP_Application();

    static IEEE1451_NCAP_Application *get_instance();

    TIM_Cache *get_tim_cache(const std::string &address);
//#ifdef USE_SIP
 //   TIM_Cache *get_tim_cache_sip(const std::string &uri);
//#endif
    IEEE1451_TEDS_Retriever *get_retriever(unsigned short trans_id);
    IEEE1451_TEDS_Retriever *get_retriever(const std::string &address);

    void update_tim(const std::string &address);
    void update_tim_completed(IEEE1451_TEDS_Retriever *retriever, IEEE1451_TIM_Channel *tim, const std::string &address);

    void report_tim_connected(const std::string &address);
    void report_tim_disconnected(const std::string &address);
    void report_command_reply(const std::string &address, unsigned short trans_id, const char *message, unsigned int length);
    void report_tim_initiated_message(const std::string &address, const char *message, unsigned int length);

    void read_temperature(TIM_Cache *tim_cache, unsigned long offset, const char *buffer, unsigned short length);
    void read_audio(TIM_Cache *tim_cache, unsigned long offset, const char *buffer, unsigned short length);

    unsigned short send_operate(const std::string &address, unsigned short channel_number) {
        return IEEE1451_NCAP::get_instance()->send_command(address, channel_number, COMMAND_CLASS_TRANSDUCER_CHANNEL_OPERATE);
    }

    unsigned short send_idle(const std::string &address, unsigned short channel_number) {
        return IEEE1451_NCAP::get_instance()->send_command(address, channel_number, COMMAND_CLASS_TRANSDUCER_CHANNEL_IDLE);
    }

    unsigned short send_read_teds(const std::string &address, unsigned short channel_number, char tedsId) {
        return IEEE1451_NCAP::get_instance()->send_command(address, channel_number, COMMAND_CLASS_READ_TEDS_SEGMENT, &tedsId, 1);
    }

    unsigned short send_read_data_set(const std::string &address, unsigned short channel_number, bool multimedia = false) {
        unsigned int offset = 0;
        return IEEE1451_NCAP::get_instance()->send_command(address, channel_number, COMMAND_CLASS_READ_TRANSDUCER_CHANNEL_DATA_SET_SEGMENT, (char *) &offset, sizeof(offset), multimedia);
    }

    unsigned short send_start_read_data_set(const std::string &address, unsigned short channel_number) {
        return IEEE1451_NCAP::get_instance()->send_command(address, channel_number, COMMAND_CLASS_START_READ_TRANSDUCER_CHANNEL_DATA_SET_SEGMENT);
    }

    unsigned short send_stop_read_data_set(const std::string &address, unsigned short channel_number) {
        return IEEE1451_NCAP::get_instance()->send_command(address, channel_number, COMMAND_CLASS_STOP_READ_TRANSDUCER_CHANNEL_DATA_SET_SEGMENT);
    }

/*#ifdef USE_SIP
    void send_sip_invite(SIP_User_Agent *ua);
    void send_sip_bye(SIP_User_Agent *ua);
    void send_sip_message(SIP_User_Agent *ua, const std::string &data);
    void send_sip_notify(SIP_User_Agent *ua, SIP_Subscription_State state, SIP_Pidf_Xml_Basic_Element pidfXml);
    void send_sip_invite_response(SIP_User_Agent *ua, unsigned short status_code, const std::string remote);
    static int message_callback(SIP_Event_Callback event, SIP_User_Agent *ua, const std::string remote);
#else*/
    static void test_read_data_set_thread(TIM_Cache *tim_cache);
//#endif

    void execute() {
        IEEE1451_NCAP::get_instance()->execute();
    }

private:
    std::list<TIM_Cache *> _cache;
    std::list<IEEE1451_TEDS_Retriever *> _retrievers;

    static IEEE1451_NCAP_Application *_application;
};


class IEEE1451_TEDS_Retriever
{
private:
    friend class IEEE1451_NCAP_Application;

    enum State
    {
        meta_teds = 0,
        tim_transducer_name_teds = 1,
        phy_teds = 2,
        transducer_channel_teds = 3,
        transducer_name_teds = 4,
        units_extension_teds = 5
    };

public:
    IEEE1451_TEDS_Retriever(const std::string &address, IEEE1451_NCAP_Application *application);
    //~IEEE1451_TEDS_Retriever() {}

    //void repeat() { execute(); }
    void process(const char *message, unsigned int length);
    void execute();

private:
    IEEE1451_NCAP_Application *_application;
    IEEE1451_Transducer_Channel *_transducer;
    IEEE1451_TIM_Channel *_tim;

    short _state;
    char _teds_id;
    unsigned short _last_trans_id;
};

//-------------------------------------------

IEEE1451_NCAP_Application *IEEE1451_NCAP_Application::_application = 0;

IEEE1451_NCAP_Application::IEEE1451_NCAP_Application()
{
    IEEE1451_NCAP::get_instance()->set_application(this);
/*
#ifdef USE_SIP
    SIP_Manager::get_instance()->init();
    SIP_Manager::register_user_handler(message_callback);
#endif*/
}

IEEE1451_NCAP_Application::~IEEE1451_NCAP_Application()
{
    std::list<TIM_Cache *>::iterator it1 = _cache.begin();
    while (it1 != _cache.end())
    {
        TIM_Cache *tim_cache = *it1++;
        _cache.pop_front();
        delete tim_cache;
    }

    std::list<IEEE1451_TEDS_Retriever *>::iterator it2 = _retrievers.begin();
    while (it2 != _retrievers.end())
    {
        IEEE1451_TEDS_Retriever *retriever = *it2++;
        _retrievers.pop_front();
        delete retriever->_tim; //This TIM is not in cache yet!
        delete retriever;
    }
}

IEEE1451_NCAP_Application *IEEE1451_NCAP_Application::get_instance()
{
    if (!_application)
        _application = new IEEE1451_NCAP_Application();
    return _application;
}
/*
#ifdef USE_SIP

IEEE1451_NCAP_Application::TIM_Cache::TIM_Cache(IEEE1451_TIM_Channel *tim, SIP_User_Agent *ua) :
    _tim(tim), _ua(ua), _rtp(0)
{
    bool multimedia = tim->_transducer->_physical_unit == TEDS_TRANSDUCER_CHANNEL_AUDIO_STREAM;

    if (multimedia)
    {
        _audio_buffer = new short[tim->_transducer->_repeats];
        _law = '1'; //A-law //_law = '2'; //PCM
        _rate = 2; //2 bits per sample (16 kbit/s)
        _reset = 1; //Yes
    }else
        _audio_buffer = 0;

    _rtp_mutex = create_mutex();
}

#else*/

IEEE1451_NCAP_Application::TIM_Cache::TIM_Cache(IEEE1451_TIM_Channel *tim) :
    _tim(tim), _read_thread(0)
{
    bool multimedia = tim->_transducer->_physical_unit == TEDS_TRANSDUCER_CHANNEL_AUDIO_STREAM;

    if (multimedia)
    {
        _audio_buffer = new short[tim->_transducer->_repeats];
        _law = '1'; //A-law //_law = '2'; //PCM
        _rate = 2; //2 bits per sample (16 kbit/s)
        _reset = 1; //Yes
    }else
        _audio_buffer = 0;
}

//#endif

IEEE1451_NCAP_Application::TIM_Cache::~TIM_Cache()
{
    delete _tim;

    if (_audio_buffer)
        delete _audio_buffer;

/*#ifdef USE_SIP
    delete _ua;

    if (_rtp)
        delete _rtp;

    delete_mutex(_rtp_mutex);
#endif*/
}

IEEE1451_NCAP_Application::TIM_Cache *IEEE1451_NCAP_Application::get_tim_cache(const std::string &address)
{
    std::list<TIM_Cache *>::iterator it = _cache.begin();
    while (it != _cache.end())
    {
        TIM_Cache *cache = *it++;
        if (cache->_tim->_address == address)
            return cache;
    }

    return 0;
}
/*
#ifdef USE_SIP

IEEE1451_NCAP_Application::TIM_Cache *IEEE1451_NCAP_Application::get_tim_cache_sip(const std::string &uri)
{
    std::list<TIM_Cache *>::iterator it = _cache.begin();
    while (it != _cache.end())
    {
        TIM_Cache *tim_cache = *it++;

        if (tim_cache->_ua->get_uri() == uri)
            return tim_cache;
    }

    return 0;
}

#endif
*/
IEEE1451_TEDS_Retriever *IEEE1451_NCAP_Application::get_retriever(unsigned short trans_id)
{
    std::list<IEEE1451_TEDS_Retriever *>::iterator it = _retrievers.begin();
    while (it != _retrievers.end())
    {
        IEEE1451_TEDS_Retriever *retriever = *it++;
        if (retriever->_last_trans_id == trans_id)
            return retriever;
    }

    return 0;
}

IEEE1451_TEDS_Retriever *IEEE1451_NCAP_Application::get_retriever(const std::string &address)
{
    std::list<IEEE1451_TEDS_Retriever *>::iterator it = _retrievers.begin();
    while (it != _retrievers.end())
    {
        IEEE1451_TEDS_Retriever *retriever = *it++;
        if (retriever->_tim->_address == address)
            return retriever;
    }

    return 0;
}

void IEEE1451_NCAP_Application::update_tim(const std::string &address)
{
    IEEE1451_TEDS_Retriever *retriever = get_retriever(address);
    if (!retriever)
    {
        retriever = new IEEE1451_TEDS_Retriever(address, this);
        _retrievers.push_back(retriever);
    }else
        std::cout << "Retriever already exist in update_tim\n";
    retriever->execute();
}

void IEEE1451_NCAP_Application::update_tim_completed(IEEE1451_TEDS_Retriever *retriever, IEEE1451_TIM_Channel *tim, const std::string &address)
{/*
#ifdef USE_SIP
    std::string uri = "sip:";
    uri += retriever->_transducer->_name;
    uri += "@";
    uri += get_address_string();

    TIM_Cache *tim_cache = new TIM_Cache(tim, SIP_Manager::get_instance()->create_user_agent(uri));
#else*/
    TIM_Cache *tim_cache = new TIM_Cache(tim);
//#endif

    _cache.push_back(tim_cache);

    _retrievers.remove(retriever);
    //delete retriever; //TODO: Descomentando causa erro! Por que?

    report_tim_connected(address);
/*
#ifndef USE_SIP
    if ((tim_cache->_tim->_transducer->_data_xmit == DATA_XMIT_COMMANDED) ||
        (tim_cache->_tim->_transducer->_data_xmit == DATA_XMIT_COMMANDED_BUFFER) ||
        (tim_cache->_tim->_transducer->_data_xmit == DATA_XMIT_COMMANDED_INTERVAL) ||
        (tim_cache->_tim->_transducer->_data_xmit == DATA_XMIT_COMMANDED_BUFFER_INTERVAL))
    {
        std::cout << "Polling sensor connected (address=" << address << ")\n";
        tim_cache->_read_thread = start_thread((void *) test_read_data_set_thread, tim_cache);
    }
#endif*/
}

void IEEE1451_NCAP_Application::report_tim_connected(const std::string &address)
{
    TIM_Cache *tim_cache = get_tim_cache(address);

    if (!tim_cache)
        update_tim(address);
    else
    {
        std::cout << "++ TIM CONNECTED (address=" << address << ") ++\n";
        tim_cache->_tim->_connected = true;
/*
#ifdef USE_SIP
        if (tim_cache->_ua->has_subscription())
            send_sip_notify(tim_cache->_ua, SIP_SUBSCRIPTION_STATE_ACTIVE, SIP_PIDF_XML_OPEN);
#endif
*/
        send_operate(address, tim_cache->_tim->_transducer->_channel_number);
    }
}

void IEEE1451_NCAP_Application::report_tim_disconnected(const std::string &address)
{
    TIM_Cache *tim_cache = get_tim_cache(address);
    if (tim_cache)
    {
        tim_cache->_tim->_connected = false;
/*
#ifdef USE_SIP
        if (tim_cache->_ua->has_subscription())
            send_sip_notify(tim_cache->_ua, SIP_SUBSCRIPTION_STATE_ACTIVE, SIP_PIDF_XML_CLOSED);
#endif
*/
        std::cout << "++ TIM DISCONNECTED (address=" << address << ") ++\n";
    }
}

void IEEE1451_NCAP_Application::report_command_reply(const std::string &address, unsigned short trans_id, const char *message, unsigned int length)
{
    IEEE1451_TEDS_Retriever *retriever = get_retriever(address);

    if (retriever)
    {
        IEEE1451_TEDS_Read_Reply *reply = (IEEE1451_TEDS_Read_Reply *) message;
        const char *buffer = message + sizeof(IEEE1451_TEDS_Read_Reply);

        if ((!reply->_header._success) || (reply->_header._length < sizeof(reply->_offset)))
        {
            std::cout << "Error in report command reply (teds)\n";
            return;
        }

        std::cout << "++ REPLY RECEIVED (teds) (trans_id=" << trans_id << ", address=" << address << ", len=" << length << ", hdr_len=" << reply->_header._length << ") ++\n";

        retriever->process(buffer, length - sizeof(IEEE1451_TEDS_Read_Reply));
    }else
    {
        IEEE1451_Data_Set_Read_Reply *reply = (IEEE1451_Data_Set_Read_Reply *) message;
        const char *buffer = message + sizeof(IEEE1451_Data_Set_Read_Reply);
        unsigned short buffer_len = reply->_header._length - sizeof(reply->_offset);

        if ((!reply->_header._success) || (reply->_header._length < sizeof(reply->_offset)))
        {
            std::cout << "Error in report command reply (data set)\n";
            return;
        }

        //std::cout << "++ REPLY RECEIVED (data set) (trans_id=" << trans_id << ", address=" << address << ", length=" << reply->_header._length << ") ++\n";

        TIM_Cache *tim_cache = get_tim_cache(address);
        if (!tim_cache)
            return;

        if (tim_cache->_tim->_transducer->_physical_unit == TEDS_TRANSDUCER_CHANNEL_KELVINS)
            read_temperature(tim_cache, reply->_offset, buffer, buffer_len);

        else if (tim_cache->_tim->_transducer->_physical_unit == TEDS_TRANSDUCER_CHANNEL_AUDIO_STREAM)
            read_audio(tim_cache, reply->_offset, buffer, buffer_len);
    }
}

void IEEE1451_NCAP_Application::report_tim_initiated_message(const std::string &address, const char *message, unsigned int length)
{
    IEEE1451_Command *cmd = (IEEE1451_Command *) message;
    unsigned long offset = *(unsigned long *) (message + sizeof(IEEE1451_Command));
    const char *buffer = message + sizeof(IEEE1451_Command) + sizeof(unsigned long);

    //std::cout << "++ TIM MESSAGE RECEIVED (address=" << address << ", cmd=0x" << std::hex << cmd->_command << ") ++\n";

    TIM_Cache *tim_cache = get_tim_cache(address);
    if (!tim_cache)
        return;

    if ((cmd->_channel_number == 1) && (cmd->_command == COMMAND_CLASS_READ_TRANSDUCER_CHANNEL_DATA_SET_SEGMENT))
    {
        if (tim_cache->_tim->_transducer->_physical_unit == TEDS_TRANSDUCER_CHANNEL_KELVINS)
        {
            std::cout << "++ TIM MESSAGE RECEIVED (address=" << address << ", cmd=0x" << std::hex << cmd->_command << ") ++\n";

            read_temperature(tim_cache, offset, buffer, cmd->_length);

        }else if (tim_cache->_tim->_transducer->_physical_unit == TEDS_TRANSDUCER_CHANNEL_AUDIO_STREAM)
            read_audio(tim_cache, offset, buffer, cmd->_length);

    }else if ((cmd->_channel_number == 1) && (cmd->_command == COMMAND_CLASS_START_READ_TRANSDUCER_CHANNEL_DATA_SET_SEGMENT))
    {
        std::cout << "++ TIM MESSAGE RECEIVED (address=" << address << ", cmd=0x" << std::hex << cmd->_command << ") ++\n";
/*
#ifdef USE_SIP
        if (!tim_cache->_ua->connected())
            send_sip_invite(tim_cache->_ua);
#endif*/
    }else if ((cmd->_channel_number == 1) && (cmd->_command == COMMAND_CLASS_STOP_READ_TRANSDUCER_CHANNEL_DATA_SET_SEGMENT))
    {
        std::cout << "++ TIM MESSAGE RECEIVED (address=" << address << ", cmd=0x" << std::hex << cmd->_command << ") ++\n";
/*
#ifdef USE_SIP
        if (tim_cache->_ua->connected())
            send_sip_bye(tim_cache->_ua);
#endif*/
    }
}

void IEEE1451_NCAP_Application::read_temperature(TIM_Cache *tim_cache, unsigned long offset, const char *buffer, unsigned short length)
{
    IEEE1451_TEDS_NCAP *channel = tim_cache->_tim->_transducer->get_teds(TEDS_TRANSDUCER_CHANNEL);

    char *sample_str = channel->get_tlv(TEDS_TRANSDUCER_CHANNEL_SAMPLE);
    IEEE1451_TEDS_NCAP *sample = new IEEE1451_TEDS_NCAP(sample_str[0], &sample_str[2], sample_str[1], true);

    char *data_model = sample->get_tlv(TEDS_TRANSDUCER_CHANNEL_DAT_MODEL);
    unsigned short model = data_model[2];

    delete sample;

    if (model == 1) //Model = 1 -> Float32: Single-precision real
    {
        std::string data;

        char aux[11];
        itoa(offset, aux);
        data = aux;
        data += " ";

        for (unsigned short i = 0, j = 0; i < tim_cache->_tim->_transducer->_repeats; i++, j += 4)
        {
            float value = *((float *)(&buffer[j]));

            itoa((int) value, aux);
            data += aux;
            if (i != (tim_cache->_tim->_transducer->_repeats - 1))
                data += ",";
        }

        std::cout << "Read temperature (" << std::dec << tim_cache->_tim->_transducer->_repeats << "): " << data << "\n";
/*
#ifdef USE_SIP
        if (tim_cache->_ua->has_subscription())
            send_sip_message(tim_cache->_ua, data);
#endif*/
    }
}
/*
void IEEE1451_NCAP_Application::read_audio(TIM_Cache *tim_cache, unsigned long offset, const char *buffer, unsigned short length)
{
    static unsigned short global_counter = 0;

    if (offset == 0)
        global_counter = 0;
    else
    {
        if (global_counter != offset)
        {
            std::cout << "Error getting audio (expected=" << std::dec << global_counter << ", received=" << std::dec << offset << ")\n";
            global_counter = offset;
        }
    }
    global_counter++;

    std::cout << "Read audio (" << std::dec << std::dec << tim_cache->_tim->_transducer->_repeats << ", " << std::dec << offset << ")\n";

//static int start = -1, end = -1, packet5s = 135;
//if ((offset % packet5s == 0) || (start == -1))
//{
//    start = get_tick();
//    std::cout << "start: " << start << " ms\n";
//}
//if (offset % packet5s == (packet5s - 1))
//{
//    end = get_tick();
//    std::cout << "end: " << end << " ms\n";
//    std::cout << "time: " << (end - start) << " ms\n";
//    end = start = -1;
//}

    for (unsigned int i = 0, j = 0; j < (tim_cache->_tim->_transducer->_repeats / 4); i += 4, j++)
    {
//std::cout << (short) buffer[j] << " ";
//if ((j + 1) % 20 == 0)
//    std::cout << "\n";

        tim_cache->_audio_buffer[i] = (buffer[j] >> 6) & 0x03;
        tim_cache->_audio_buffer[i + 1] = (buffer[j] >> 4) & 0x03;
        tim_cache->_audio_buffer[i + 2] = (buffer[j] >> 2) & 0x03;
        tim_cache->_audio_buffer[i + 3] = buffer[j] & 0x03;

//std::cout << tim_cache->_audio_buffer[i] << " " << tim_cache->_audio_buffer[i + 1] << " "
//          << tim_cache->_audio_buffer[i + 2] << " " << tim_cache->_audio_buffer[i + 3] << "(" << (short) buffer[j] << ") ";
//if ((j + 1) % 4 == 0)
//    std::cout << "\n";
    }

    G726_decode(tim_cache->_audio_buffer, tim_cache->_audio_buffer, tim_cache->_tim->_transducer->_repeats,
                &tim_cache->_law, tim_cache->_reset, tim_cache->_rate, &tim_cache->_state);
    tim_cache->_reset = 0;

    /*static FILE *file = fopen("/home/leonardo/EPOS/ieee1451-ncap/record/recordPCM.sw", "wb");
    //static FILE *file2 = fopen("/home/leonardo/EPOS/ieee1451-ncap/record/recordPCM.txt", "w");
    if (file)
    {
        if (tim_cache->_law == '2')
        {
            for (unsigned int i = 0; i < tim_cache->_tim->_transducer->_repeats; i++)
            {
                //fwrite(&tim_cache->_audio_buffer[i], 2, 1, file);
                fwrite(&tim_cache->_audio_buffer[i], 2, 1, file);

                //if (file2)
                //{
                //    char v[11];
                //    itoa(tim_cache->_audio_buffer[i], v);
                //    fputs(v, file2);

                //    //std::cout << tim_cache->_audio_buffer[i] << "(" << v << ") ";
                //    if ((i + 1) % 15 == 0)
                //    {
                //        //std::cout << "\n";
                //        fputs("\n", file2);
                //    }else
                //        fputs(" ", file2);
                //}
            }
            //fwrite(tim_cache->_audio_buffer, 2, tim_cache->_tim->_transducer->_repeats, file);
        }
    }*/
/*
    char *audio = new char[tim_cache->_tim->_transducer->_repeats];
    for (unsigned int i = 0; i < tim_cache->_tim->_transducer->_repeats; i++)
        audio[i] = (char) tim_cache->_audio_buffer[i];

    /*static FILE *file = fopen("/home/leonardo/EPOS/ieee1451-ncap/record/recordA.al", "wb");
    if (file)
    {
        //if (tim_cache->_law == '2')
        //    fwrite(tim_cache->_audio_buffer, 2, tim_cache->_tim->_transducer->_repeats, file);
        //else if (tim_cache->_law == '1')
        //{
            fwrite(audio, 1, tim_cache->_tim->_transducer->_repeats, file);
            //for (unsigned int i = 0; i < tim_cache->_tim->_transducer->_repeats; i++)
            //    fwrite(&tim_cache->_audio_buffer[i], 1, 1, file);
        //}
    }
    //fclose(file);*/
/*
#ifdef USE_SIP
    lock_mutex(tim_cache->_rtp_mutex);

    //TODO: Está ignorando os pacotes de áudio antes de receber a confirmação de conexão.
    if ((tim_cache->_ua->connected()) && (tim_cache->_rtp))
        tim_cache->_rtp->add_payload(audio, tim_cache->_tim->_transducer->_repeats);

    unlock_mutex(tim_cache->_rtp_mutex);
#endif

    delete audio;
}

#ifdef USE_SIP

void IEEE1451_NCAP_Application::send_sip_invite(SIP_User_Agent *ua)
{
    std::string remote = ua->get_subscriber();
    if (remote.empty())
    {
        std::cout << "IEEE1451_NCAP_Application::send_sip_invite -> No subscriber address\n";
        return;
    }

    SIP_Request *invite = ua->create_invite(remote);
    if (!invite)
    {
        std::cout << "IEEE1451_NCAP_Application::send_sip_invite -> Failed to create INVITE request\n";
        return;
    }

    ua->send_request(invite);
    delete invite;
}

void IEEE1451_NCAP_Application::send_sip_bye(SIP_User_Agent *ua)
{
    std::string remote = ua->get_subscriber();
    if (remote.empty())
    {
        std::cout << "IEEE1451_NCAP_Application::send_sip_bye -> No subscriber address\n";
        return;
    }

    SIP_Request *bye = ua->create_bye(remote);
    if (!bye)
    {
        std::cout << "IEEE1451_NCAP_Application::send_sip_bye -> Failed to create BYE request\n";
        return;
    }

    ua->send_request(bye);
    delete bye;
}

void IEEE1451_NCAP_Application::send_sip_message(SIP_User_Agent *ua, const std::string &data)
{
    std::string remote = ua->get_subscriber();
    if (remote.empty())
    {
        std::cout << "IEEE1451_NCAP_Application::send_sip_message -> No subscriber address\n";
        return;
    }

    SIP_Request *message = ua->create_message(remote, data);
    if (!message)
    {
        std::cout << "IEEE1451_NCAP_Application::send_sip_message -> Failed to create MESSAGE request\n";
        return;
    }

    ua->send_request(message);
    delete message;
}

void IEEE1451_NCAP_Application::send_sip_notify(SIP_User_Agent *ua, SIP_Subscription_State state, SIP_Pidf_Xml_Basic_Element pidfXml)
{
    std::string remote = ua->get_subscriber();
    if (remote.empty())
    {
        std::cout << "IEEE1451_NCAP_Application::send_sip_notify -> No subscriber address\n";
        return;
    }

    SIP_Request *notify = ua->create_notify(remote, state, pidfXml, 3600);
    if (!notify)
    {
        std::cout << "IEEE1451_NCAP_Application::send_sip_notify -> Failed to create NOTIFY request\n";
        return;
    }

    ua->send_request(notify);
    delete notify;
}

void IEEE1451_NCAP_Application::send_sip_invite_response(SIP_User_Agent *ua, unsigned short status_code, const std::string remote)
{
    ua->send_invite_response(status_code, remote);
}

int IEEE1451_NCAP_Application::message_callback(SIP_Event_Callback event, SIP_User_Agent *ua, const std::string remote)
{
    IEEE1451_NCAP_Application *app = IEEE1451_NCAP_Application::get_instance();
    TIM_Cache *tim_cache = app->get_tim_cache_sip(ua->get_uri());
    if (!tim_cache)
    {
        std::cout << "++ Invalid User Agent ++\n";
        return 1;
    }

    switch (event)
    {
        case SIP_SESSION_INVITATION:
        {
            std::cout << "++ Session Invitation (" << remote << ") ++\n";

            bool multimedia = tim_cache->_tim->_transducer->_physical_unit == TEDS_TRANSDUCER_CHANNEL_AUDIO_STREAM;
            if (multimedia)
            {
                std::cout << "++ Session Invitation Accepted ++\n";
                app->send_sip_invite_response(ua, 200, remote);
            }else
            {
                std::cout << "++ Session Invitation Rejected ++\n";
                app->send_sip_invite_response(ua, 480, remote);
            }
            break;
        }

        case SIP_SESSION_INITIATED:
        {
            std::cout << "++ Session Initiated (" << remote << ", " << tim_cache->_ua->get_call_status() << ") ++\n";

            const SIP_Session *session = tim_cache->_ua->get_session();

            lock_mutex(tim_cache->_rtp_mutex);

            if (tim_cache->_rtp)
                delete tim_cache->_rtp;

            tim_cache->_rtp = new RTP_Adapter();
            tim_cache->_rtp->init(session->_local_address, session->_local_port,
                                  session->_remote_address, session->_remote_port,
                                  RTP::PCMA, 160, true, false);

            unlock_mutex(tim_cache->_rtp_mutex);

            if (tim_cache->_ua->get_call_status() == SIP_CALL_STATUS_INCOMING)
                IEEE1451_NCAP_Application::get_instance()->send_start_read_data_set(tim_cache->_tim->_address, 0x01);
            break;
        }

        case SIP_SESSION_TERMINATED:
        {
            std::cout << "++ Session Terminated (" << remote << ") ++\n";

            if (tim_cache->_ua->get_call_status() == SIP_CALL_STATUS_INCOMING)
                IEEE1451_NCAP_Application::get_instance()->send_stop_read_data_set(tim_cache->_tim->_address, 0x01);

            lock_mutex(tim_cache->_rtp_mutex);

            if (tim_cache->_rtp)
            {
                delete tim_cache->_rtp;
                tim_cache->_rtp = 0;
            }

            unlock_mutex(tim_cache->_rtp_mutex);
            break;
        }

        case SIP_SUBSCRIPTION_INITIATED:
        {
            std::cout << "++ Subscription Initiated (" << remote << ") ++\n";

            if (tim_cache->_tim->_connected)
                app->send_sip_notify(ua, SIP_SUBSCRIPTION_STATE_ACTIVE, SIP_PIDF_XML_OPEN);
            else
                app->send_sip_notify(ua, SIP_SUBSCRIPTION_STATE_ACTIVE, SIP_PIDF_XML_CLOSED);
            break;
        }

        case SIP_SUBSCRIPTION_TERMINATED:
            std::cout << "++ Subscription Terminated (" << remote << ") ++\n";
            break;

        case SIP_MESSAGE_RECEIVED:
        {
            std::cout << "++ Message Received (" << remote << ") ++\n";

            std::string message = ua->get_text_received();
            size_t found;

            if (((found = message.find("Read TransducerChannel data-set segment ")) != std::string::npos) &&
                ((tim_cache->_tim->_transducer->_data_xmit == DATA_XMIT_COMMANDED) ||
                 (tim_cache->_tim->_transducer->_data_xmit == DATA_XMIT_COMMANDED_BUFFER) ||
                 (tim_cache->_tim->_transducer->_data_xmit == DATA_XMIT_COMMANDED_INTERVAL) ||
                 (tim_cache->_tim->_transducer->_data_xmit == DATA_XMIT_COMMANDED_BUFFER_INTERVAL)) &&
                 (tim_cache->_tim->_transducer->_physical_unit == TEDS_TRANSDUCER_CHANNEL_KELVINS))
            {
                std::string aux = message.substr(found + strlen("Read TransducerChannel data-set segment "));
                unsigned long offset = (unsigned short) atol(aux.c_str());

                //TODO: Offset está sendo ignorado!
                std::cout << "++ Read TransducerChannel data-set segment (offset=" << offset << ") ++\n";
                app->send_read_data_set(tim_cache->_tim->_address, 0x01);

            }else if ((found = message.find("Read TEDS segment ")) != std::string::npos)
            {
                std::string aux = message.substr(found + strlen("Read TEDS segment "));
                unsigned short dst = (unsigned short) atol(aux.c_str());
                if ((found = aux.find_first_of(" ")) != std::string::npos)
                {
                    aux = aux.substr(found + 1);
                    unsigned short code = (unsigned short) atol(aux.c_str());
                    if ((found = aux.find_first_of(" ")) != std::string::npos)
                    {
                        aux = aux.substr(found + 1);
                        unsigned long offset = (unsigned long) atol(aux.c_str());

                        //TODO: Offset está sendo ignorado!
                        std::cout << "++ Read TEDS segment (dst=" << dst << ", code=" << code << ", offset=" << offset << ") ++\n";
                        app->send_read_teds(tim_cache->_tim->_address, dst, code);
                    }
                }
            }else
                std::cout << "++ Invalid Message Received ++\n";
            break;
        }

        default:
            break;
    }

    return 0;
}

#else
*/
void IEEE1451_NCAP_Application::test_read_data_set_thread(TIM_Cache *tim_cache)
{
    bool multimedia_sensor = tim_cache->_tim->_transducer->_physical_unit == TEDS_TRANSDUCER_CHANNEL_AUDIO_STREAM;
    bool optimized_polling = true;

    std::cout << "Test read data set thread created (address=" << tim_cache->_tim->_address << ", multimedia=" << multimedia_sensor << ")\n";

    if (!multimedia_sensor)
    {
        while (true)
        {
         //   Alarm::delay(16000); //500 * 20 * 10);
            std::cout << "NCAP requesting data (address=" << tim_cache->_tim->_address << ")\n";

            if (tim_cache->_tim->_connected)
                IEEE1451_NCAP_Application::get_instance()->send_read_data_set(tim_cache->_tim->_address, 0x01);
            else
                std::cout << "TIM disconnected\n";
        }
    }else
    {
        if (!optimized_polling)
        {
            while (true)
            {
             //   Alarm::delay(5000); //50 * 20 * 10);
                std::cout << "NCAP requesting data (address=" << tim_cache->_tim->_address << ")\n";

                if (tim_cache->_tim->_connected)
                {
                    std::cout << "Starting polling (address=" << tim_cache->_tim->_address << ")\n";
                    unsigned int start = 2000;//Alarm::get_tick();

                    for (unsigned short i = 0; i < 27 * 5; i++)
                    {
                        IEEE1451_NCAP_Application::get_instance()->send_read_data_set(tim_cache->_tim->_address, 0x01, true);
                     //   delay(37);
                    }

                    unsigned int end = 3000;//get_tick();
                    std::cout << "Polling time: " << (end - start) << " ms\n";
                    std::cout << "Stopping polling (address=" << tim_cache->_tim->_address << ")\n";
                }else
                    std::cout << "TIM disconnected\n";
            }
        }else
        {
            while (true)
            {
             //   Alarm::delay(5000); //50 * 20 * 10);
                std::cout << "NCAP requesting data (address=" << tim_cache->_tim->_address << ")\n";

                if (tim_cache->_tim->_connected)
                {
                    std::cout << "Sending start read data set (address=" << tim_cache->_tim->_address << ")\n";
                    IEEE1451_NCAP_Application::get_instance()->send_start_read_data_set(tim_cache->_tim->_address, 0x01);

                   // delay(5170); //delay(10330);

                    std::cout << "Sending stop read data set (address=" << tim_cache->_tim->_address << ")\n";
                    IEEE1451_NCAP_Application::get_instance()->send_stop_read_data_set(tim_cache->_tim->_address, 0x01);
                }else
                    std::cout << "TIM disconnected\n";
            }
        }
    }
}

//#endif

//-------------------------------------------

IEEE1451_TEDS_Retriever::IEEE1451_TEDS_Retriever(const std::string &address, IEEE1451_NCAP_Application *application) :
    _application(application)
{
    _transducer = new IEEE1451_Transducer_Channel();
    _transducer->_address = address;
    _transducer->_channel_number = 0x01; //It just supports one transducer per mote!

    _tim = new IEEE1451_TIM_Channel();
    _tim->_address = address;
    _tim->_transducer = _transducer;

    _state = meta_teds;
    _last_trans_id = 0;
}

void IEEE1451_TEDS_Retriever::process(const char *message, unsigned int length)
{
    switch (_state)
    {
        case meta_teds:
        case tim_transducer_name_teds:
        case phy_teds:
        {
            //IEEE1451_TEDS_NCAP *teds = new IEEE1451_TEDS_NCAP(_teds_id, message, length);
            //_tim->add_teds(teds);
            break;
        }

        case units_extension_teds:
        {
            //IEEE1451_TEDS_NCAP *teds = new IEEE1451_TEDS_NCAP(_teds_id, message, length);
            //_transducer->add_teds(teds);
            break;
        }

        case transducer_name_teds:
        {
            IEEE1451_TEDS_NCAP *teds = new IEEE1451_TEDS_NCAP(_teds_id, message, length);
            //_transducer->add_teds(teds);

            char *tc_name = teds->get_tlv(TEDS_USER_TRANSDUCER_NAME_TC_NAME);
            unsigned short tc_name_len = tc_name[1];

            _transducer->_name.assign(&tc_name[2], tc_name_len);

            delete teds;
            break;
        }

        case transducer_channel_teds:
        {
            IEEE1451_TEDS_NCAP *teds = new IEEE1451_TEDS_NCAP(_teds_id, message, length);
            _transducer->add_teds(teds);

            char *phy_str = teds->get_tlv(TEDS_TRANSDUCER_CHANNEL_PHY_UNITS);
            IEEE1451_TEDS_NCAP *phy = new IEEE1451_TEDS_NCAP(phy_str[0], &phy_str[2], phy_str[1], true);

            char *kelvin = phy->get_tlv(TEDS_TRANSDUCER_CHANNEL_KELVINS);
            char *audio = phy->get_tlv(TEDS_TRANSDUCER_CHANNEL_AUDIO_STREAM);

            if (kelvin)
                _transducer->_physical_unit = TEDS_TRANSDUCER_CHANNEL_KELVINS;
            if (audio)
                _transducer->_physical_unit = TEDS_TRANSDUCER_CHANNEL_AUDIO_STREAM;

            delete phy;


            char *data_set_str = teds->get_tlv(TEDS_TRANSDUCER_CHANNEL_DATA_SET);
            IEEE1451_TEDS_NCAP *data_set = new IEEE1451_TEDS_NCAP(data_set_str[0], &data_set_str[2], data_set_str[1], true);

            char *max_data_repetitions = data_set->get_tlv(TEDS_TRANSDUCER_CHANNEL_REPEATS);
            _transducer->_repeats = (((unsigned char) max_data_repetitions[2]) << 8) | ((unsigned char) max_data_repetitions[3]);

            delete data_set;


            char *data_xmit = teds->get_tlv(TEDS_TRANSDUCER_CHANNEL_DATA_XMIT);
            _transducer->_data_xmit = data_xmit[2];
            break;
        }

        default:
            break;
    }

    _state++;
    execute();
}

void IEEE1451_TEDS_Retriever::execute()
{
    unsigned short channel_number = 0xff;

    switch (_state)
    {
        case meta_teds:
            std::cout << ">> Getting meta teds...\n";
            _teds_id = TEDS_META;
            channel_number = 0x00;
            break;

        case tim_transducer_name_teds:
            std::cout << ">> Getting tim's transducer name teds...\n";
            _teds_id = TEDS_USER_TRANSDUCER_NAME;
            channel_number = 0x00;
            break;

        case phy_teds:
            std::cout << ">> Getting phy teds...\n";
            _teds_id = TEDS_PHY;
            channel_number = 0x00;
            break;

        case transducer_channel_teds:
            std::cout << ">> Getting transducer channel teds...\n";
            _teds_id = TEDS_TRANSDUCER_CHANNEL;
            channel_number = 0x01;
            break;

        case transducer_name_teds:
            std::cout << ">> Getting transducer's transducer name teds...\n";
            _teds_id = TEDS_USER_TRANSDUCER_NAME;
            channel_number = 0x01;
            break;

        case units_extension_teds:
            if (_transducer->_physical_unit == TEDS_TRANSDUCER_CHANNEL_AUDIO_STREAM)
            {
                std::cout << ">> Getting transducer's units extension teds...\n";
                _teds_id = TEDS_UNITS_EXTENSION;
                channel_number = 0x01;
                break;
            }

        default:
            std::cout << ">> TEDS completed...\n";
            _teds_id = 0;
            _application->update_tim_completed(this, _tim, _tim->_address);
            return;
    }

    _last_trans_id = _application->send_read_teds(_tim->_address, channel_number, _teds_id);
}

//-------------------------------------------

int main()
{
    std::cout << "+++++ Starting ncap +++++\n";

    IEEE1451_NCAP_Application *app = IEEE1451_NCAP_Application::get_instance();
    app->execute();
    return 0;
}
